@@ -4,8 +4,11 @@ from dynaconf.base import LazySettings  # noqa
 from dynaconf.constants import DEFAULT_SETTINGS_FILES
 from dynaconf.contrib import DjangoDynaconf  # noqa
 from dynaconf.contrib import FlaskDynaconf  # noqa
+from dynaconf.utils.inspect import get_history
 from dynaconf.utils.inspect import inspect_settings
 from dynaconf.utils.parse_conf import add_converter  # noqa
+from dynaconf.utils.parse_conf import DynaconfFormatError  # noqa
+from dynaconf.utils.parse_conf import DynaconfParseError  # noqa
 from dynaconf.validator import ValidationError  # noqa
 from dynaconf.validator import Validator  # noqa
 
@@ -33,4 +36,7 @@ __all__ = [
     "DjangoDynaconf",
     "add_converter",
     "inspect_settings",
+    "get_history",
+    "DynaconfFormatError",
+    "DynaconfParseError",
 ]
