@@ -1,12 +1,15 @@
 from __future__ import annotations
 
-import io
 import json
 import os
+import sys
 from collections import namedtuple
+from pathlib import Path
+from textwrap import dedent
 
 import pytest
 
+from dynaconf import add_converter
 from dynaconf import default_settings
 from dynaconf import Dynaconf
 from dynaconf.loaders.json_loader import DynaconfEncoder
@@ -21,6 +24,7 @@ from dynaconf.utils import trimmed_split
 from dynaconf.utils import upperfy
 from dynaconf.utils.files import find_file
 from dynaconf.utils.files import get_local_filename
+from dynaconf.utils.parse_conf import boolean_fix
 from dynaconf.utils.parse_conf import evaluate_lazy_format
 from dynaconf.utils.parse_conf import Formatters
 from dynaconf.utils.parse_conf import Lazy
@@ -482,3 +486,36 @@ def test_env_list():
         "other",
         "global",
     ]
+
+
+def create_file(filename: str, data: str) -> str:
+    with open(filename, "w", encoding="utf-8") as f:
+        f.write(dedent(data))
+    return filename
+
+
+@pytest.mark.skipif(
+    sys.platform.startswith("win"),
+    reason="Doesn't work on windows due to backslash decoding errors",
+)
+def test_add_converter_path_example(tmp_path):
+    """Assert add_converter Path example works"""
+    add_converter("path", Path)
+    fn = create_file(
+        tmp_path / "settings.yaml",
+        f"""\
+        my_path: "@path {Path.home()}"
+        """,
+    )
+    settings = Dynaconf(settings_file=fn)
+    assert isinstance(settings.my_path, Path)
+
+
+def test_boolean_fix():
+    """Assert boolean fix works"""
+    assert boolean_fix("True") == "true"
+    assert boolean_fix("False") == "false"
+    assert boolean_fix("NotOnlyTrue") == "NotOnlyTrue"
+    assert boolean_fix("TrueNotOnly") == "TrueNotOnly"
+    assert boolean_fix("FalseNotOnly") == "FalseNotOnly"
+    assert boolean_fix("NotOnlyFalse") == "NotOnlyFalse"
